"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loggingWS = exports.getColoredText = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const router_js_1 = require("./router.js");
const routerUtils_js_1 = require("./routerUtils.js");
const colors_js_1 = require("./colors.js");
const weblogging_js_1 = require("./weblogging.js");
let sendNewLogToClient = function (props) { };
const loggingValues = {
    fatal: 0,
    error: 1,
    warn: 2,
    important: 3,
    success: 4,
    info: 5,
    verbose: 6
};
function getColoredText(text, color = 'black', ...notColoredTexts) {
    // const colors: { [key in Tcolor]: ChalkInstance } = {
    const coloredText = colors_js_1.colors[color](text) + notColoredTexts;
    return coloredText;
}
exports.getColoredText = getColoredText;
/**
* @param {String} text text to return
* @param {*} color enums: [black, red, green, yellow, blue, magenta, cyan, white]
* @returns colored text
*/
function coloredConsoleLog(text, color = 'black', ...notColoredTexts) {
    const coloredText = getColoredText(text, color, ...notColoredTexts);
    console.log(coloredText);
}
class OurLogging extends weblogging_js_1.WebServerLogging {
    loggingBrowserLength;
    historyLogsLength;
    #logLevel = 6;
    #logFileName;
    constructor(historyLogsLength) {
        super();
        const defaultLength = 100;
        this.loggingBrowserLength = 80;
        this.historyLogsLength = historyLogsLength || defaultLength;
    }
    setLogLevel(newLevel) { this.#logLevel = loggingValues[newLevel]; }
    setBrowserMessagesAmount(amount) { this.loggingBrowserLength = amount; }
    setLogfileName(name) { this.#logFileName = name; }
    fatal(text, stackTrace = '') {
        if (!this.#aboveSelectedLogLevel('fatal'))
            return;
        const currentTime = this.#getDateTime();
        let logText = `\n[ FATAL ERROR ] ${currentTime} | ${text}  ${stackTrace}`;
        coloredConsoleLog('[ FATAL ERROR ] ', 'carmine', `${currentTime} | ${text} ${stackTrace}`);
        this.#addLogToGlobal(text + ' ' + stackTrace, 'fatal');
        this.#logTextToFile(logText);
    }
    error(text, stackTrace = '') {
        if (!this.#aboveSelectedLogLevel('error'))
            return;
        const currentTime = this.#getDateTime();
        let logText = `\n[ ERROR ] ${currentTime} | ${text} ${stackTrace}`;
        coloredConsoleLog('[ ERROR ] ', 'scarlet', `${currentTime} | ${text} ${stackTrace}`);
        this.#addLogToGlobal(text + ' ' + stackTrace, 'error');
        this.#logTextToFile(logText);
    }
    warn(...text) {
        if (!this.#aboveSelectedLogLevel('warn'))
            return;
        const currentTime = this.#getDateTime();
        let logText = `\n[ WARNING ] ${currentTime} | ${text.join(' ')}`;
        coloredConsoleLog('[ WARNING ] ', 'yellow', `${currentTime} | ${text.join(' ')}`);
        this.#addLogToGlobal(text.join(' '), 'warn');
        this.#logTextToFile(logText);
    }
    important(...text) {
        if (!this.#aboveSelectedLogLevel('important'))
            return;
        const currentTime = this.#getDateTime();
        let logText = `\n[ IMPORTANT ] ${currentTime} | ${text.join(' ')}`;
        coloredConsoleLog('[ IMPORTANT ] ', 'blue', `${currentTime} | ${text.join(' ')}`);
        this.#addLogToGlobal(text.join(' '), 'info');
        this.#logTextToFile(logText);
    }
    info(...text) {
        if (!this.#aboveSelectedLogLevel('info'))
            return;
        const currentTime = this.#getDateTime();
        let logText = `\n[ INFO ] ${currentTime} | ${text.join(' ')}`;
        coloredConsoleLog('[ INFO ] ', 'lightblue', `${currentTime} | ${text.join(' ')}`);
        this.#addLogToGlobal(text.join(' '), 'info');
        this.#logTextToFile(logText);
    }
    verbose(...text) {
        if (!this.#aboveSelectedLogLevel('verbose'))
            return;
        const currentTime = this.#getDateTime();
        let logText = `\n[ VERBOSE ] ${currentTime} | ${text.join(' ')}`;
        coloredConsoleLog('[ VERBOSE ] ', 'verbose_color', `${currentTime} | ${text.join(' ')}`);
        this.#addLogToGlobal(text.join(' '), 'verbose');
        this.#logTextToFile(logText);
    }
    success(...text) {
        if (!this.#aboveSelectedLogLevel('success'))
            return;
        const currentTime = this.#getDateTime();
        let logText = `\n[ SUCCESS ] ${currentTime} | ${text.join(' ')}`;
        coloredConsoleLog('[ SUCCESS ] ', 'green', `${currentTime} | ${text.join(' ')}`);
        this.#addLogToGlobal(text.join(' '), 'success');
        this.#logTextToFile(logText);
    }
    #addLogToGlobal(text, type) {
        if (router_js_1.loggingLocal.length > this.loggingBrowserLength)
            router_js_1.loggingLocal.shift();
        router_js_1.loggingLocal.push({ text, type, date: new Date().getTime() });
        sendNewLogToClient({ text, type, date: new Date().getTime() });
    }
    #getDateTime() {
        var currentDate = new Date();
        const date = currentDate.toLocaleDateString('en-US');
        const time = currentDate.toLocaleTimeString('en-US');
        return `${date} | ${time}`;
    }
    #logTextToFile(logText) {
        if (!this.#logFileName)
            return;
        if (!fs_1.default.existsSync(this.#logFileName)) {
            const dirname = path_1.default.dirname(this.#logFileName);
            fs_1.default.mkdirSync(dirname, { recursive: true });
        }
        try {
            fs_1.default.appendFileSync(this.#logFileName, logText);
        }
        catch (error) {
            console.error(coloredConsoleLog('LOGGING ERROR: ', 'scarlet', error.message));
        }
    }
    #aboveSelectedLogLevel(testedLogLevel) {
        const selectedLogLevel = this.#logLevel;
        const wantedLogLevel = loggingValues[testedLogLevel];
        return wantedLogLevel <= selectedLogLevel;
    }
}
const logging = new OurLogging();
exports.default = logging;
__exportStar(require("./router.js"), exports);
const loggingWS = (ws) => {
    sendNewLogToClient = function (props) {
        ws.send(JSON.stringify({
            newLog: (0, routerUtils_js_1.getSingleLog)(props.type, props.date, props.text)
        }));
    };
};
exports.loggingWS = loggingWS;
